package javax.imageio.metadata;

import org.w3c.dom.UserDataHandler;
import org.w3c.dom.TypeInfo;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import java.util.ArrayList;
import java.util.List;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;

public class IIOMetadataNode implements Element, NodeList
{
    private String nodeName;
    private String nodeValue;
    private Object userObject;
    private IIOMetadataNode parent;
    private int numChildren;
    private IIOMetadataNode firstChild;
    private IIOMetadataNode lastChild;
    private IIOMetadataNode nextSibling;
    private IIOMetadataNode previousSibling;
    private List attributes;
    
    public IIOMetadataNode() {
        this.nodeName = null;
        this.nodeValue = null;
        this.userObject = null;
        this.parent = null;
        this.numChildren = 0;
        this.firstChild = null;
        this.lastChild = null;
        this.nextSibling = null;
        this.previousSibling = null;
        this.attributes = new ArrayList();
    }
    
    public IIOMetadataNode(String nodeName) {
        this.nodeName = null;
        this.nodeValue = null;
        this.userObject = null;
        this.parent = null;
        this.numChildren = 0;
        this.firstChild = null;
        this.lastChild = null;
        this.nextSibling = null;
        this.previousSibling = null;
        this.attributes = new ArrayList();
        this.nodeName = nodeName;
    }
    
    private void checkNode(Node node) throws DOMException {
        if (node == null) {
            return;
        }
        if (!(node instanceof IIOMetadataNode)) {
            throw new IIODOMException((short)4, "Node not an IIOMetadataNode!");
        }
        return;
    }
    
    @Override
    public String getNodeName() {
        return this.nodeName;
    }
    
    @Override
    public String getNodeValue() {
        return this.nodeValue;
    }
    
    @Override
    public void setNodeValue(String nodeValue) {
        this.nodeValue = nodeValue;
    }
    
    @Override
    public short getNodeType() {
        return 1;
    }
    
    @Override
    public Node getParentNode() {
        return this.parent;
    }
    
    @Override
    public NodeList getChildNodes() {
        return this;
    }
    
    @Override
    public Node getFirstChild() {
        return this.firstChild;
    }
    
    @Override
    public Node getLastChild() {
        return this.lastChild;
    }
    
    @Override
    public Node getPreviousSibling() {
        return this.previousSibling;
    }
    
    @Override
    public Node getNextSibling() {
        return this.nextSibling;
    }
    
    @Override
    public NamedNodeMap getAttributes() {
        return new IIONamedNodeMap(this.attributes);
    }
    
    @Override
    public Document getOwnerDocument() {
        return null;
    }
    
    @Override
    public Node insertBefore(Node node, Node node2) {
        if (node == null) {
            throw new IllegalArgumentException("newChild == null!");
        }
        this.checkNode(node);
        this.checkNode(node2);
        IIOMetadataNode iioMetadataNode = (IIOMetadataNode)node;
        IIOMetadataNode iioMetadataNode2 = (IIOMetadataNode)node2;
        IIOMetadataNode previousSibling;
        IIOMetadataNode nextSibling;
        if (node2 == null) {
            previousSibling = this.lastChild;
            nextSibling = null;
            this.lastChild = iioMetadataNode;
        }
        else {
            previousSibling = iioMetadataNode2.previousSibling;
            nextSibling = iioMetadataNode2;
        }
        if (previousSibling != null) {
            previousSibling.nextSibling = iioMetadataNode;
        }
        if (nextSibling != null) {
            nextSibling.previousSibling = iioMetadataNode;
        }
        iioMetadataNode.parent = this;
        iioMetadataNode.previousSibling = previousSibling;
        iioMetadataNode.nextSibling = nextSibling;
        if (this.firstChild == iioMetadataNode2) {
            this.firstChild = iioMetadataNode;
        }
        ++this.numChildren;
        return iioMetadataNode;
    }
    
    @Override
    public Node replaceChild(Node node, Node node2) {
        if (node == null) {
            throw new IllegalArgumentException("newChild == null!");
        }
        this.checkNode(node);
        this.checkNode(node2);
        IIOMetadataNode iioMetadataNode = (IIOMetadataNode)node;
        IIOMetadataNode iioMetadataNode2 = (IIOMetadataNode)node2;
        IIOMetadataNode previousSibling = iioMetadataNode2.previousSibling;
        IIOMetadataNode nextSibling = iioMetadataNode2.nextSibling;
        if (previousSibling != null) {
            previousSibling.nextSibling = iioMetadataNode;
        }
        if (nextSibling != null) {
            nextSibling.previousSibling = iioMetadataNode;
        }
        iioMetadataNode.parent = this;
        iioMetadataNode.previousSibling = previousSibling;
        iioMetadataNode.nextSibling = nextSibling;
        if (this.firstChild == iioMetadataNode2) {
            this.firstChild = iioMetadataNode;
        }
        if (this.lastChild == iioMetadataNode2) {
            this.lastChild = iioMetadataNode;
        }
        iioMetadataNode2.parent = null;
        iioMetadataNode2.previousSibling = null;
        iioMetadataNode2.nextSibling = null;
        return iioMetadataNode2;
    }
    
    @Override
    public Node removeChild(Node node) {
        if (node == null) {
            throw new IllegalArgumentException("oldChild == null!");
        }
        this.checkNode(node);
        IIOMetadataNode iioMetadataNode = (IIOMetadataNode)node;
        IIOMetadataNode previousSibling = iioMetadataNode.previousSibling;
        IIOMetadataNode nextSibling = iioMetadataNode.nextSibling;
        if (previousSibling != null) {
            previousSibling.nextSibling = nextSibling;
        }
        if (nextSibling != null) {
            nextSibling.previousSibling = previousSibling;
        }
        if (this.firstChild == iioMetadataNode) {
            this.firstChild = nextSibling;
        }
        if (this.lastChild == iioMetadataNode) {
            this.lastChild = previousSibling;
        }
        iioMetadataNode.parent = null;
        iioMetadataNode.previousSibling = null;
        iioMetadataNode.nextSibling = null;
        --this.numChildren;
        return iioMetadataNode;
    }
    
    @Override
    public Node appendChild(Node node) {
        if (node == null) {
            throw new IllegalArgumentException("newChild == null!");
        }
        this.checkNode(node);
        return this.insertBefore(node, null);
    }
    
    @Override
    public boolean hasChildNodes() {
        return this.numChildren > 0;
    }
    
    @Override
    public Node cloneNode(boolean b) {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode(this.nodeName);
        iioMetadataNode.setUserObject(this.getUserObject());
        if (b) {
            for (IIOMetadataNode iioMetadataNode2 = this.firstChild; iioMetadataNode2 != null; iioMetadataNode2 = iioMetadataNode2.nextSibling) {
                iioMetadataNode.appendChild(iioMetadataNode2.cloneNode(true));
            }
            return iioMetadataNode;
        }
        return iioMetadataNode;
    }
    
    @Override
    public void normalize() {
    }
    
    @Override
    public boolean isSupported(String s, String s2) {
        return false;
    }
    
    @Override
    public String getNamespaceURI() throws DOMException {
        return null;
    }
    
    @Override
    public String getPrefix() {
        return null;
    }
    
    @Override
    public void setPrefix(String s) {
    }
    
    @Override
    public String getLocalName() {
        return this.nodeName;
    }
    
    @Override
    public String getTagName() {
        return this.nodeName;
    }
    
    @Override
    public String getAttribute(String s) {
        Attr attributeNode = this.getAttributeNode(s);
        if (attributeNode == null) {
            return "";
        }
        return attributeNode.getValue();
    }
    
    @Override
    public String getAttributeNS(String s, String s2) {
        return this.getAttribute(s2);
    }
    
    @Override
    public void setAttribute(String s, String s2) {
        boolean n = true;
        char[] charArray = s.toCharArray();
        int i = 0;
        while (i < charArray.length) {
            if (charArray[i] >= '\ufffe') {
                n = false;
                break;
            }
            ++i;
        }
        if (n == 0) {
            throw new IIODOMException((short)5, "Attribute name is illegal!");
        }
        this.removeAttribute(s, false);
        this.attributes.add(new IIOAttr(this, s, s2));
        return;
    }
    
    @Override
    public void setAttributeNS(String s, String s2, String s3) {
        this.setAttribute(s2, s3);
    }
    
    @Override
    public void removeAttribute(String s) {
        this.removeAttribute(s, true);
    }
    
    private void removeAttribute(String s, boolean b) {
        int size = this.attributes.size();
        int i = 0;
        while (i < size) {
            IIOAttr iioAttr = (IIOAttr)this.attributes.get(i);
            if (s.equals(iioAttr.getName())) {
                iioAttr.setOwnerElement(null);
                this.attributes.remove(i);
                return;
            }
            ++i;
        }
        if (b) {
            throw new IIODOMException((short)8, "No such attribute!");
        }
        return;
    }
    
    @Override
    public void removeAttributeNS(String s, String s2) {
        this.removeAttribute(s2);
    }
    
    @Override
    public Attr getAttributeNode(String s) {
        return (Attr)this.getAttributes().getNamedItem(s);
    }
    
    @Override
    public Attr getAttributeNodeNS(String s, String s2) {
        return this.getAttributeNode(s2);
    }
    
    @Override
    public Attr setAttributeNode(Attr attr) throws DOMException {
        Element ownerElement = attr.getOwnerElement();
        if (ownerElement != null) {
            if (ownerElement == this) {
                return null;
            }
            throw new DOMException((short)10, "Attribute is already in use");
        }
        else {
            IIOAttr iioAttr;
            if ((attr instanceof IIOAttr)) {
                iioAttr = (IIOAttr)attr;
                iioAttr.setOwnerElement(this);
            }
            else {
                iioAttr = new IIOAttr(this, attr.getName(), attr.getValue());
            }
            Attr attributeNode = this.getAttributeNode(iioAttr.getName());
            if (attributeNode != null) {
                this.removeAttributeNode(attributeNode);
            }
            this.attributes.add(iioAttr);
            return attributeNode;
        }
    }
    
    @Override
    public Attr setAttributeNodeNS(Attr attributeNode) {
        return this.setAttributeNode(attributeNode);
    }
    
    @Override
    public Attr removeAttributeNode(Attr attr) {
        this.removeAttribute(attr.getName());
        return attr;
    }
    
    @Override
    public NodeList getElementsByTagName(String s) {
        ArrayList<Object> list = new ArrayList();
        this.getElementsByTagName(s, list);
        return new IIONodeList(list);
    }
    
    private void getElementsByTagName(String anObject, List list) {
        if (this.nodeName.equals(anObject)) {
            list.add(this);
        }
        for (Node node = this.getFirstChild(); node != null; node = node.getNextSibling()) {
            ((IIOMetadataNode)node).getElementsByTagName(anObject, list);
        }
    }
    
    @Override
    public NodeList getElementsByTagNameNS(String s, String s2) {
        return this.getElementsByTagName(s2);
    }
    
    @Override
    public boolean hasAttributes() {
        return this.attributes.size() > 0;
    }
    
    @Override
    public boolean hasAttribute(String s) {
        return this.getAttributeNode(s) != null;
    }
    
    @Override
    public boolean hasAttributeNS(String s, String s2) {
        return this.hasAttribute(s2);
    }
    
    @Override
    public int getLength() {
        return this.numChildren;
    }
    
    @Override
    public Node item(int n) {
        if (n < 0) {
            return null;
        }
        Node node;
        for (node = this.getFirstChild(); node != null && n-- > 0; node = node.getNextSibling()) {}
        return node;
    }
    
    public Object getUserObject() {
        return this.userObject;
    }
    
    public void setUserObject(Object userObject) {
        this.userObject = userObject;
    }
    
    @Override
    public void setIdAttribute(String s, boolean b) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public void setIdAttributeNS(String s, String s2, boolean b) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public void setIdAttributeNode(Attr attr, boolean b) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public TypeInfo getSchemaTypeInfo() throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public Object setUserData(String s, Object o, UserDataHandler userDataHandler) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public Object getUserData(String s) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public Object getFeature(String s, String s2) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public boolean isSameNode(Node node) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public boolean isEqualNode(Node node) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public String lookupNamespaceURI(String s) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public boolean isDefaultNamespace(String s) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public String lookupPrefix(String s) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public String getTextContent() throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public void setTextContent(String s) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public short compareDocumentPosition(Node node) throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
    
    @Override
    public String getBaseURI() throws DOMException {
        throw new DOMException((short)9, "Method not supported");
    }
}
